@@ -6,7 +6,11 @@ use strict;
 use warnings;
 use 5.14.2;
 
-use JSON::Validator "joi";
+use JSON::Validator::Joi;
+
+sub joi {
+    return JSON::Validator::Joi->new;
+}
 
 sub new {
     my ( $type ) = @_;
@@ -64,8 +68,8 @@ sub queue {
 sub test_id {
     return joi->string->regex('^[0-9]$|^[1-9][0-9]{1,8}$|^[0-9a-f]{16}$');
 }
-sub translation_language {
-    return joi->string->regex('^[a-zA-Z0-9-_.@]{1,30}$');
+sub language_tag {
+    return joi->string->regex('^[a-z]{2}(_[A-Z]{2})?$');
 }
 sub username {
     return joi->string->regex('^[a-zA-Z0-9-.@]{1,50}$');
